==> locust-2.17.1.dev60.tar.gz: locust-2.17.1.dev60/locust/webui/src/components/LineChart/LineChart.tsx <==
import { useEffect, useState, useRef } from 'react';
import {
  init,
  registerTheme,
  dispose,
  ECharts,
  EChartsOption,
  TooltipComponentFormatterCallbackParams,
  DefaultLabelFormatterCallbackParams,
} from 'echarts';

import { IUiState } from 'redux/slice/ui.slice';
import { ICharts } from 'types/ui.types';

interface ILine {
  name: string;
  key: keyof ICharts;
}

interface ICreateOptions {
  title: string;
  seriesData: EChartsOption['Series'][];
  charts: ICharts;
}

export interface ILineChartProps {
  title: string;
  lines: ILine[];
}

interface ILineChart extends ILineChartProps {
  title: string;
  lines: ILine[];
  charts: ICharts;
}

const createOptions = ({ charts, title, seriesData }: ICreateOptions) => ({
  legend: {
    icon: 'circle',
    inactiveColor: '#b3c3bc',
    textStyle: {
      color: '#b3c3bc',
    },
  },
  title: {
    text: title,
    x: 10,
    y: 10,
  },
  tooltip: {
    trigger: 'axis',
    formatter: (params: TooltipComponentFormatterCallbackParams) => {
      if (
        !!params &&
        Array.isArray(params) &&
        params.length > 0 &&
        params.some(param => !!param.value)
      ) {
        return params.reduce(
          (tooltipText, { color, seriesName, value }) => `
          ${tooltipText}
          <br>
          <span style="color:${color};">
            ${seriesName}:&nbsp${value}
          </span>
        `,
          '',
        );
      } else {
        return 'No data';
      }
    },
    axisPointer: {
      animation: true,
    },
    textStyle: {
      color: '#b3c3bc',
      fontSize: 13,
    },
    backgroundColor: 'rgba(21,35,28, 0.93)',
    borderWidth: 0,
    extraCssText: 'z-index:1;',
  },
  xAxis: {
    type: 'category',
    splitLine: {
      show: false,
    },
    axisLine: {
      lineStyle: {
        color: '#5b6f66',
      },
    },
    data: charts.time,
  },
  yAxis: {
    type: 'value',
    boundaryGap: [0, '5%'],
    splitLine: {
      show: false,
    },
    axisLine: {
      lineStyle: {
        color: '#5b6f66',
      },
    },
  },
  series: seriesData,
  grid: { x: 60, y: 70, x2: 40, y2: 40 },
  color: ['#00ca5a', '#ff6d6d'],
  toolbox: {
    feature: {
      saveAsImage: {
        name: title.replace(/\s+/g, '_').toLowerCase() + '_' + new Date().getTime() / 1000,
        title: 'Download as PNG',
        emphasis: {
          iconStyle: {
            textPosition: 'left',
          },
        },
      },
    },
  },
});

const getSeriesData = ({ charts, lines }: { charts: IUiState['charts']; lines: ILine[] }) =>
  lines.map(({ key, name }) => ({
    name,
    type: 'line',
    showSymbol: true,
    data: charts[key],
  }));

const createMarkLine = (charts: ICharts) => ({
  symbol: 'none',
  label: {
    formatter: (params: DefaultLabelFormatterCallbackParams) => `Run #${params.dataIndex + 1}`,
  },
  lineStyle: { color: '#5b6f66' },
  data: (charts.markers || []).map((timeMarker: string) => ({ xAxis: timeMarker })),
});

registerTheme('locust', {
  backgroundColor: '#27272a',
  xAxis: { lineColor: '#f00' },
  textStyle: { color: '#b3c3bc' },
  title: {
    textStyle: { color: '#b3c3bc' },
  },
});

export default function LineChart({ charts, title, lines }: ILineChart) {
  const [chart, setChart] = useState<ECharts | null>(null);

  const chartContainer = useRef<HTMLDivElement | null>(null);

  useEffect(() => {
    if (!chartContainer.current) {
      return;
    }

    const initChart = init(chartContainer.current, 'locust');
    initChart.setOption(
      createOptions({ charts, title, seriesData: getSeriesData({ charts, lines }) }),
    );

    setChart(initChart);

    return () => {
      dispose(initChart);
    };
  }, [chartContainer]);

  useEffect(() => {
    const isChartDataDefined = lines.every(({ key }) => !!charts[key]);
    if (chart && isChartDataDefined) {
      chart.setOption({
        xAxis: { data: charts.time },
        series: lines.map(({ key }, index) => ({
          data: charts[key],
          ...(index === 0 ? { markLine: createMarkLine(charts) } : {}),
        })),
      });
    }
  }, [charts, chart, lines]);

  return <div ref={chartContainer} style={{ width: '100%', height: '300px' }}></div>;
}

==> locust-2.19.2.dev55.tar.gz: locust-2.19.2.dev55/locust/webui/src/components/LineChart/LineChart.tsx <==
import { useEffect, useState, useRef } from 'react';
import {
  init,
  registerTheme,
  dispose,
  ECharts,
  EChartsOption,
  DefaultLabelFormatterCallbackParams,
  connect,
  TooltipComponentOption,
} from 'echarts';

import { IUiState } from 'redux/slice/ui.slice';
import { ICharts } from 'types/ui.types';

interface ILine {
  name: string;
  key: keyof ICharts;
}

interface ICreateOptions {
  title: string;
  seriesData: EChartsOption['Series'][];
  charts: ICharts;
  colors?: string[];
}

export interface ILineChartProps {
  title: string;
  lines: ILine[];
  colors?: string[];
}

interface ILineChart extends ILineChartProps {
  charts: ICharts;
}

const CHART_TEXT_COLOR = '#b3c3bc';
const CHART_AXIS_COLOR = '#5b6f66';
const CHART_COLOR_PALETTE = ['#00ca5a', '#ffca5a'];

registerTheme('locust', {
  color: CHART_COLOR_PALETTE,
  backgroundColor: '#27272a',
  xAxis: { lineColor: '#f00' },
  graph: {
    color: CHART_COLOR_PALETTE,
  },
  textStyle: { color: CHART_TEXT_COLOR },
  title: {
    textStyle: { color: CHART_TEXT_COLOR },
  },
});

const createOptions = ({ charts, title, seriesData, colors }: ICreateOptions) => ({
  legend: {
    icon: 'circle',
    inactiveColor: CHART_TEXT_COLOR,
    textStyle: {
      color: CHART_TEXT_COLOR,
    },
  },
  title: {
    text: title,
    x: 10,
    y: 10,
  },
  tooltip: {
    trigger: 'axis',
    formatter: (params: TooltipComponentOption) => {
      if (
        !!params &&
        Array.isArray(params) &&
        params.length > 0 &&
        params.some(param => !!param.value)
      ) {
        return params.reduce(
          (tooltipText, { axisValue, color, seriesName, value }, index) => `
          ${index === 0 ? axisValue : ''}
          ${tooltipText}
          <br>
          <span style="color:${color};">
            ${seriesName}:&nbsp${value}
          </span>
        `,
          '',
        );
      } else {
        return 'No data';
      }
    },
    axisPointer: {
      animation: true,
    },
    textStyle: {
      color: CHART_TEXT_COLOR,
      fontSize: 13,
    },
    backgroundColor: 'rgba(21,35,28, 0.93)',
    borderWidth: 0,
    extraCssText: 'z-index:1;',
  },
  xAxis: {
    type: 'category',
    splitLine: {
      show: false,
    },
    axisLine: {
      lineStyle: {
        color: CHART_AXIS_COLOR,
      },
    },
    data: charts.time,
  },
  yAxis: {
    type: 'value',
    boundaryGap: [0, '5%'],
    splitLine: {
      show: false,
    },
    axisLine: {
      lineStyle: {
        color: CHART_AXIS_COLOR,
      },
    },
  },
  series: seriesData,
  grid: { x: 60, y: 70, x2: 40, y2: 40 },
  color: colors,
  toolbox: {
    feature: {
      saveAsImage: {
        name: title.replace(/\s+/g, '_').toLowerCase() + '_' + new Date().getTime() / 1000,
        title: 'Download as PNG',
        emphasis: {
          iconStyle: {
            textPosition: 'left',
          },
        },
      },
    },
  },
});

const getSeriesData = ({ charts, lines }: { charts: IUiState['charts']; lines: ILine[] }) =>
  lines.map(({ key, name }) => ({
    name,
    type: 'line',
    showSymbol: true,
    data: charts[key],
  }));

const createMarkLine = (charts: ICharts) => ({
  symbol: 'none',
  label: {
    formatter: (params: DefaultLabelFormatterCallbackParams) => `Run #${params.dataIndex + 1}`,
  },
  lineStyle: { color: CHART_AXIS_COLOR },
  data: (charts.markers || []).map((timeMarker: string) => ({ xAxis: timeMarker })),
});

export default function LineChart({ charts, title, lines, colors }: ILineChart) {
  const [chart, setChart] = useState<ECharts | null>(null);

  const chartContainer = useRef<HTMLDivElement | null>(null);

  useEffect(() => {
    if (!chartContainer.current) {
      return;
    }

    const initChart = init(chartContainer.current, 'locust');
    initChart.setOption(
      createOptions({ charts, title, seriesData: getSeriesData({ charts, lines }), colors }),
    );

    const handleChartResize = () => initChart.resize();
    window.addEventListener('resize', handleChartResize);

    initChart.group = 'swarmCharts';
    connect('swarmCharts');

    setChart(initChart);

    return () => {
      dispose(initChart);
      window.removeEventListener('resize', handleChartResize);
    };
  }, [chartContainer]);

  useEffect(() => {
    const isChartDataDefined = lines.every(({ key }) => !!charts[key]);
    if (chart && isChartDataDefined) {
      chart.setOption({
        xAxis: { data: charts.time },
        series: lines.map(({ key }, index) => ({
          data: charts[key],
          ...(index === 0 ? { markLine: createMarkLine(charts) } : {}),
        })),
      });
    }
  }, [charts, chart, lines]);

  return <div ref={chartContainer} style={{ width: '100%', height: '300px' }}></div>;
}
